#include <SD.h>
#include <SPI.h>
#include <SoftwareSerial.h>

SoftwareSerial mySerial(PIN_PC6,PIN_PC7); // RX, TX

#define F_CPU 8000000UL  
#define SS PIN_PB4
#define MOSI PIN_PB5
#define MISO PIN_PB6
#define SCK PIN_PB7
#define nCE1 PIN_PB0
#define nCE2 PIN_PB2
#define nMR PIN_PB1
#define PGM PIN_PB3
#define WRITE PIN_PA0
#define LED1 PIN_PC0
#define LED2 PIN_PC1
#define LED3 PIN_PC2
File dataFile;

uint16_t addr = 0; //шина адреса
uint8_t dat = 0; //шина данных
uint8_t cdat = 0; //шина данных - контроль
uint8_t pgm_count1 = 0; // счетчик попыток записи одного байта в EPROM1
uint8_t pgm_count2 = 0; // счетчик попыток записи одного байта в EPROM2
uint16_t bed_write1 = 0; // счетчик неудачных попыток записи в EPROM1
uint16_t bed_write2 = 0; // счетчик неудачных попыток записи в EPROM2

void setup() {

delay(1000);  
DDRA = B11111110; // устанавливает все выводы как выходные кроме младшего бита, который при старте используется для ввода режима
// после чего это будет шина адреса h_addr //DDRA = B11111111; // устанавливает все выводы как выходные - это шина адреса h_addr
DDRB = B11111111; // устанавливает все выводы как выходные - это шина управления
DDRC = B11111111; // устанавливает все выводы как выходные - это шина адреса l_addr
DDRD = B00000000; // устанавливает все выводы как входные - это шина данных, будет изменена в режиме записи
pinMode (nCE1, OUTPUT); // /CE1
pinMode (nCE2, OUTPUT); // /CE2
pinMode (nMR, OUTPUT); // //MR
pinMode (PGM, OUTPUT); // PGM
pinMode (WRITE, INPUT); // режим чтение\запись
PORTC = B00000000; // ноль на порту С

digitalWrite(nCE1, HIGH);
digitalWrite(nCE2, HIGH);
digitalWrite(nMR, LOW); // сразу блокируем кроме режима программирования
digitalWrite(PGM, LOW);

  // сбрасываем регистры платы
  digitalWrite(nMR, LOW);
  digitalWrite(nMR, HIGH);
  
 if (!SD.begin(SS)) {
    // горит левый если карты нет
    digitalWrite(LED1, HIGH); 
    digitalWrite(LED2, LOW);
    digitalWrite(LED3, LOW);
    while(1);
  }

}
void loop() {
  
  // Выбор режима, если 0, то режим чтения, если 1, то режим записи
  if (digitalRead(WRITE) == LOW) { 
    /////////////////////////////////////////////////////////////
    // Режим чтения  
  DDRA = B11111111; // все ноги порта А на вывод
  // Чтение EPROM1
  if (SD.exists("eprom1.bin")) SD.remove("eprom1.bin"); // удаляем старый файл на карте
  dataFile = SD.open("eprom1.bin", FILE_WRITE); // создаем новый файл на карте    
  addr=0;
readloop1:      
  PORTC = addr & 0xff; // выставляем младший байт адреса
  PORTA = addr >> 8; // выставляем старший байт адреса
  digitalWrite(nCE1, LOW); // выставляем разрешение на чтение EPROM1
  delayMicroseconds(1);
  dat = PIND; // читаем содержимое ячейки из EPROM1
  delayMicroseconds(1);
  digitalWrite(nCE1, HIGH); // выставляем запрет на чтение EPROM1
  if (dataFile) dataFile.write(dat); // отправляем считанный байт в карту
  else {
    // горит средний если не удалось открыть и записать
    digitalWrite(LED1, LOW);
    digitalWrite(LED2, HIGH);
    digitalWrite(LED3, LOW);
    while(1);
  }
  addr++;
  if (addr != 0) goto readloop1;
  dataFile.close(); // закрываем файл 

  // Чтение EPROM2
  if (SD.exists("eprom2.bin")) SD.remove("eprom2.bin"); // удаляем старый файл на карте
  dataFile = SD.open("eprom2.bin", FILE_WRITE); // создаем новый файл на карте    
addr=0;
readloop2:  
  PORTC = addr & 0xff; // выставляем младший байт адреса
  PORTA = addr >> 8; // выставляем старший байт адреса
  digitalWrite(nCE2, LOW); // выставляем разрешение на чтение EPROM2
  delayMicroseconds(1);
  dat = PIND; // читаем содержимое ячейки из EPROM2
  delayMicroseconds(1);
  digitalWrite(nCE2, HIGH); // выставляем запрет на чтение EPROM2
  if (dataFile) dataFile.write(dat); // отправляем считанный байт в карту
  else {
    // горит средний если не удалось открыть и записать
    digitalWrite(LED1, LOW);
    digitalWrite(LED2, HIGH); 
    digitalWrite(LED3, LOW);
    while(1);
  }
  addr++;
  if (addr != 0) goto readloop2;
  dataFile.close(); // закрываем файл 
 
  // finish - перемигиваются левый и правый
  digitalWrite(LED2, LOW);
  while(1) {
  digitalWrite(LED1, LOW);
  digitalWrite(LED3, HIGH);
  delay(500);
  digitalWrite(LED1, HIGH);
  digitalWrite(LED3, LOW);
  delay(500); 
  }  

  } // конец режима чтения

  else {
  /////////////////////////////////////////////////////////////
    // Режим программирования   
  DDRA = B11111111; // все ноги порта А на вывод

  // Чтение файла EPROM1
  dataFile = SD.open("eprom1.bin"); // открываем файл для чтения 
  addr=0;
writeloop1:      
  if (dataFile) dat = dataFile.read();
  else {
      // горит правый если не удалось открыть и считать в режиме программирования 
     digitalWrite(LED1, LOW);
     digitalWrite(LED2, LOW);
     digitalWrite(LED3, HIGH);
     while(1);
  }
  pgm_count1 = 0; // обнуление счетчика попыток записи
pgmloop1:
  // запись одного байта - dat
  DDRD = B11111111; // устанавливает все выводы как выходные
  digitalWrite(nMR, HIGH); // разрешаем управление этим регистром в режиме записи 
  delayMicroseconds(1);
  digitalWrite(PGM, HIGH); // подаем на плату 12,75В (1)
  delay(2);
  // т.к. подача на плату 12,75В меняет логику её работы
  PORTC = addr & 0xff; // выставляем младший байт адреса
  PORTA = addr >> 8; // выставляем старший байт адреса
  PORTD = dat; // выставляем шину данных (2)
  delayMicroseconds(1);
  digitalWrite(nCE1, LOW);
  delayMicroseconds(1); 
  digitalWrite(nCE1, HIGH); // импульс на запись адреса и данных в регистры адреса и данных платы (3)
  PORTD = B00000001; // установка 00000001 на шине данных (4)
  delayMicroseconds(1);
  digitalWrite(nCE2, LOW); 
  delayMicroseconds(1);
  digitalWrite(nCE2, HIGH); // импульс на вывод данных с регистра на EPROM1 (5)
  PORTD = B00000101; // установка 00000101 на шине данных для EPROM1 (6)
  delayMicroseconds(1);
  digitalWrite(nCE2, LOW); 
  delayMicroseconds(1);
  digitalWrite(nCE2, HIGH); // фронт подключает /OE EPROM1 (7)
  delayMicroseconds(100); // держим длительность импульса программирования 100 мкс 
  digitalWrite(nMR, LOW); // останавливаем импульс программирования (8)
  delayMicroseconds(1);
    // чтение записанного байта
  DDRD = B00000000; // устанавливает все выводы как входные  
  digitalWrite(PGM, LOW); // снимаем с платы 12,75В (9)
  // т.к. подача на плату 12,75В меняет логику её работы
  delay(2);
  PORTC = addr & 0xff; // выставляем младший байт адреса
  PORTA = addr >> 8; // выставляем старший байт адреса (10)
  delayMicroseconds(1);
  digitalWrite(nCE1, LOW); // выставляем разрешение на чтение EPROM1 (11)
  delayMicroseconds(1);
  cdat = PIND; // читаем содержимое ячейки из EPROM2
  delayMicroseconds(1);
  digitalWrite(nCE1, HIGH); // выставляем запрет на чтение EPROM2
  pgm_count1++; // 25 попыток записи каждого байта
  if (pgm_count1 > 25) { 
     // горят средний и правый если запись байта не удалась 
     digitalWrite(LED1, LOW);
     digitalWrite(LED2, HIGH); 
     digitalWrite(LED3, HIGH);
     mySerial.begin(9600);
     mySerial.println("....................................................................");
     mySerial.println("EPROM1");
     mySerial.println("адрес:  ");
     uint8_t addr_high = addr >> 8; // старший байт адреса
     mySerial.print("addr_high:  ");
     mySerial.println(addr_high,HEX);
     uint8_t addr_low = addr & 0xff; // младший байт адреса
     mySerial.print("addr_lowh:  ");
     mySerial.println(addr_low,HEX);
     mySerial.print("dat - записываемое:  ");
     mySerial.println(dat,HEX);
     mySerial.print("cdat - прочитанное:  ");
     mySerial.println(cdat,HEX);
     mySerial.println("EPROM1");
     mySerial.println("количество попыток повторной записи");
     uint8_t bed_write1_high = bed_write1 >> 8; // старший байт 
     uint8_t bed_write1_low = bed_write1 & 0xff; // младший байт 
     mySerial.println(bed_write1_high);
     mySerial.println(bed_write1_low);
     while(1);
  }
  if (cdat != dat) {
    bed_write1++; // инкрементируем счетчик неудачных повторных записей в EPROM1
    goto pgmloop1; // если записанное не равно записываемому, то новая попытка 
  }
  // если записанное равно записываемому
  addr++;
  if (addr != 0) goto writeloop1;
  dataFile.close(); // закрываем файл 

  // Чтение файла EPROM2
  dataFile = SD.open("eprom2.bin"); // открываем файл для чтения 
  addr=0;
writeloop2:      
  if (dataFile) dat = dataFile.read();
  else {
      // горит правый если не удалось открыть и считать в режиме программирования 
     digitalWrite(LED1, LOW);
     digitalWrite(LED2, LOW);
     digitalWrite(LED3, HIGH);
     while(1);
  }
  pgm_count2 = 0; // обнуление счетчика попыток записи
pgmloop2:
  // запись одного байта - dat
  DDRD = B11111111; // устанавливает все выводы как выходные
  digitalWrite(nMR, HIGH); // разрешаем управление этим регистром в режиме записи 
  delayMicroseconds(1);
  digitalWrite(PGM, HIGH); // подаем на плату 12,75В (1)
  delay(2);
  // т.к. подача на плату 12,75В меняет логику её работы
  PORTC = addr & 0xff; // выставляем младший байт адреса
  PORTA = addr >> 8; // выставляем старший байт адреса
  PORTD = dat; // выставляем шину данных (2)
  delayMicroseconds(1);
  digitalWrite(nCE1, LOW);
  delayMicroseconds(1); 
  digitalWrite(nCE1, HIGH); // импульс на запись адреса и данных в регистры адреса и данных платы (3)
  PORTD = B00000001; // установка 00000001 на шине данных (4)
  delayMicroseconds(1);
  digitalWrite(nCE2, LOW); 
  delayMicroseconds(1);
  digitalWrite(nCE2, HIGH); // импульс на вывод данных с регистра на EPROM1 (5)
  PORTD = B00000011; // установка 00000011 на шине данных для EPROM2 (6)
  delayMicroseconds(1);
  digitalWrite(nCE2, LOW); 
  delayMicroseconds(1);
  digitalWrite(nCE2, HIGH); // фронт подключает /OE EPROM2 (7)
  delayMicroseconds(100); // держим длительность импульса программирования 100 мкс 
  digitalWrite(nMR, LOW); // останавливаем импульс программирования (9)
  delayMicroseconds(1);
    // чтение записанного байта
  DDRD = B00000000; // устанавливает все выводы как входные 
  digitalWrite(PGM, LOW); // снимаем с платы 12,75В (10)
  // т.к. подача на плату 12,75В меняет логику её работы
  delay(2);
  PORTC = addr & 0xff; // выставляем младший байт адреса
  PORTA = addr >> 8; // выставляем старший байт адреса (11)
  delayMicroseconds(1);
  digitalWrite(nCE2, LOW); // выставляем разрешение на чтение EPROM2 (12)
  delayMicroseconds(1);
  cdat = PIND; // читаем содержимое ячейки из EPROM2
  delayMicroseconds(1);
  digitalWrite(nCE2, HIGH); // выставляем запрет на чтение EPROM2
  pgm_count2++; // 25 попыток записи каждого байта
  if (pgm_count2 > 25) { 
     // горят средний и правый если запись байта не удалась 
     digitalWrite(LED1, LOW);
     digitalWrite(LED2, HIGH); 
     digitalWrite(LED3, HIGH);
     mySerial.begin(9600);
     mySerial.println("....................................................................");
     mySerial.println("EPROM2");
     mySerial.println("адрес:  ");
     uint8_t addr_high = addr >> 8; // старший байт адреса
     mySerial.print("addr_high:  ");
     mySerial.println(addr_high,HEX);
     uint8_t addr_low = addr & 0xff; // младший байт адреса
     mySerial.print("addr_lowh:  ");
     mySerial.println(addr_low,HEX);
     mySerial.print("dat - записываемое:  ");
     mySerial.println(dat,HEX);
     mySerial.print("cdat - прочитанное:  ");
     mySerial.println(cdat,HEX);
     mySerial.println("EPROM2");
     uint8_t bed_write2_high = bed_write2 >> 8; // старший байт 
     uint8_t bed_write2_low = bed_write2 & 0xff; // младший байт 
     mySerial.println("количество попыток повторной записи"); 
     mySerial.println(bed_write2_high);
     mySerial.println(bed_write2_low);
     while(1);
  }
  if (cdat != dat) {
    bed_write2++; // инкрементируем счетчик неудачных повторных записей в EPROM2
    goto pgmloop2; // если записанное не равно записываемому, то новая попытка 
  }
  // если записанное равно записываемому
  addr++;
  if (addr != 0) goto writeloop2;
  dataFile.close(); // закрываем файл 

    
  // finish -  - перемигиваются средний и правый
     uint8_t bed_write1_high = bed_write1 >> 8; // старший байт 
     uint8_t bed_write1_low = bed_write1 & 0xff; // младший байт 
     uint8_t bed_write2_high = bed_write2 >> 8; // старший байт 
     uint8_t bed_write2_low = bed_write2 & 0xff; // младший байт 
     mySerial.begin(9600);
     mySerial.println("....................................................................");
     mySerial.println("EPROM");
     mySerial.println("количество попыток повторной записи");
     mySerial.println("для EPROM1:"); 
     mySerial.println(bed_write1_high);
     mySerial.println(bed_write1_low);
     mySerial.println("для EPROM2:"); 
     mySerial.println(bed_write2_high);
     mySerial.println(bed_write2_low);
          
  while(1) {
  digitalWrite(LED1, HIGH);
  digitalWrite(LED2, HIGH);
  digitalWrite(LED3, HIGH);
  delay(500);
  digitalWrite(LED1, LOW);
  digitalWrite(LED2, LOW);
  digitalWrite(LED3, LOW);
  delay(500); 
  }  

  } // конец режима программирования
 
  
  // конец программы
}
